import p5 from 'p5';
import SensorBuffer from './sensor-buffer.js';
import './style.css';

let sketch = (p) => {

  let sensors = [
    // Degrees.
    'rotationX', 'rotationY', 'rotationZ',

    // Metres per second.
    'accelerationX', 'accelerationY', 'accelerationZ',
  ];

  let buffers = {};
  let bufferLength = 20;

  let running = false;
  let orientationLocked = false;

  let dotX = p.windowWidth / 2;
  let dotY = p.windowHeight / 2;
  let dotSize = 5;
  let pDotSize = dotSize;
  let dotSpeed = 50;

  p.preload = () => {
    p.createDiv().id('app');
    p.createDiv().id('buttons').parent('app');
  }

  p.setup = () => {
    let canvas = p.createCanvas(p.windowWidth, p.windowHeight); // p.WEBGL
    canvas.parent('app');

    if (typeof(DeviceOrientationEvent) !== 'undefined') {
      if (typeof(DeviceOrientationEvent.requestPermission) === 'function') {
        // iOS 13 device.
        DeviceOrientationEvent.requestPermission()
          .catch(() => {
            // Show permission dialogue first time only.
            let orientationButton = p.createButton('Allow access to orientation sensor.');
            orientationButton.parent('buttons');
            orientationButton.mousePressed(p.requestOrientationAccess);
          })
      }
    } else {
      // Incompatible device.
      alert('Browser does not support orientation sensors')
    }

    if (typeof(DeviceMotionEvent) !== 'undefined') {
      if (typeof(DeviceMotionEvent.requestPermission) === 'function') {
        // iOS 13 device
        DeviceMotionEvent.requestPermission()
          .catch(() => {
            // Show permission dialogue first time only.
            let motionButton = p.createButton('Allow access to motion sensor.');
            motionButton.parent('buttons');
            motionButton.mousePressed(p.requestMotionAccess);
          })
      }
    } else {
      // Non-mobile device
      alert('Browser does not support motion sensors')
    }

    let runButton = p.createButton('Start');
    runButton.parent('buttons');
    runButton.mousePressed(() => {
      running = !running;
      console.log(`Run: ${running}`);

      if (running) {
        p.fullscreen(true);
        // Try locking the screen orientation.
        orientationLocked = true;
        screen.orientation.lock('portrait-primary')
          .catch((error) => {
            if (error instanceof DOMException) {
              // Cannot lock orientation, not a mobile device.
              orientationLocked = false;
            } else {
              throw error;
            }
          });
        runButton.html('Stop');
        p.loop();
      } else {
        p.fullscreen(false);
        // Release lock on screen orientation.
        if (orientationLocked) {
          screen.orientation.unlock();
          orientationLocked = false;
        };
        runButton.html('Start');
        p.noLoop();
      }
    });

    p.setMoveThreshold(0.01);
    p.noLoop();

    // Instantiate buffers for all sensors.
    sensors.forEach((sensor) => {
      buffers[sensor] = new SensorBuffer(bufferLength);
    });
  }

  p.draw = () => {
    p.background(200);

    // Push data into buffers.
    Object.entries(buffers).forEach(([sensorName, buffer]) => {
      buffer.push(p[sensorName]);
    });

    // From the point of view of the canvas, swap x and y.
    const dx = p.map(p.rotationY, -180, 180, -1, 1) * dotSpeed;
    const dy = p.map(p.rotationX, -180, 180, -1, 1) * dotSpeed;

    dotX += dx;
    dotY += dy;

    dotX = p.constrain(dotX, 0, p.width);
    dotY = p.constrain(dotY, 0, p.height);

    // Map rotationZ directly to dotsize, but smooth values.
    pDotSize = dotSize;
    dotSize = p.map2(p.rotationZ, 0, 360, 5, p.width / 3, 0.5);
    dotSize = (pDotSize + dotSize) / 2;

    p.circle(dotX, dotY, dotSize);

    let accelX = (buffers['accelerationX'].mean) * 3;
    let accelY = (buffers['accelerationY'].mean) * p.height / 2;
    let accelZ = (buffers['accelerationZ'].mean) * 2;

    let lineRotate = p.constrain(-accelX, -p.PI / 2, p.PI / 2);
    let linePosition = p.constrain(accelY, -p.height / 2, p.height / 2);
    let lineSize = p.constrain(0.5 + accelZ, 0.1, 2);
    //p.text(lineSize, 10, 10);

    p.push()
    p.translate(p.width / 2, p.height / 2);
    p.rotate(lineRotate);
    //p.scale(lineSize);
    p.line(-p.width / 2 * lineSize,
           linePosition,
           p.width / 2 * lineSize,
           linePosition);
    p.pop()

    p.textSize(10);

    let sensorX = 80;
    let sensorY = 80;
    let sensorPad = 50;

    for (let i = 0; i < sensors.length; i++) {
      let value = p.round(p[sensors[i]], 1);
      p.text(`${sensors[i]}: ${value}`, sensorX, sensorY + (sensorPad * i));
    }

  };

  p.requestOrientationAccess = () => {
    DeviceOrientationEvent.requestPermission()
      .then(permissionState => {
        if (permissionState === 'granted') {
          orientationButton.remove();
        }
      })
      .catch(console.error);
  };

  p.requestMotionAccess = () => {
    DeviceMotionEvent.requestPermission()
      .then(permissionState => {
        if (permissionState === 'granted') {
          motionButton.remove();
        }
      })
      .catch(console.error);
  };

  p.deviceMoved = () => {
    //p.background(255, 0, 0);
  }

  p.map2 = (value, start1, stop1, start2, stop2, midpoint) => {
    let v = p.norm(value, start1, stop1);

    if (v > midpoint) {
      v = midpoint - (v - midpoint);
    }

    return p.map(v, 0, midpoint, start2, stop2);
  }

}

new p5(sketch, 'app'); // FIXME: Cannot pass ID if creating div in preload!
